
#include "vkeymap.h"


#pragma VxD_LOCKED_CODE_SEG
#pragma VxD_LOCKED_DATA_SEG

// Globals

UCHAR	KeyMappings[256];
BOOL	fLoaded = FALSE;
BOOL	fMappingOk = FALSE;
BOOL	fLastExtended = FALSE;

//
//	OnDeviceInit
//
BOOL OnDeviceInit(PVMMCB hVM, DWORD refdata)
{
	HKEY	hKey;
	DWORD	rc, dwSize;

	if (fLoaded)
		return FALSE;

	// Get the registry stuff

	rc = _RegOpenKey (HKEY_LOCAL_MACHINE, VXD_KEY, &hKey); 
	
	if (rc == ERROR_SUCCESS)
		Debug_Printf ("_RegOpenKey Success!!!!\r\n");

	dwSize = sizeof(KeyMappings);

	rc = _RegQueryValueEx (hKey, "KeyMappings", 0, 0, KeyMappings, &dwSize);

	if (rc == ERROR_SUCCESS && dwSize == sizeof(KeyMappings))
	{
		Debug_Printf ("_RegQueryValueEx Success!!!!\r\n");
		fMappingOk = TRUE;
	}

	rc = _RegCloseKey (hKey);

	if (rc == ERROR_SUCCESS)
		Debug_Printf ("_RegCloseKey Success!!!!\r\n");

	fLoaded = TRUE;

    return TRUE;
}


//
//	OnInitComplete
//
BOOL OnInitComplete(PVMMCB hVM, DWORD refdata)
{
	// A bug in Windows makes us hook the keyboard here rather than
	// in the above OnDeviceInit.

	HookKeyboard();
	
    return TRUE;
}

//
//	OnDeviceIoControl
//
DWORD OnDeviceIoControl (PDIOCPARAMETERS pDIO)
{
	if (!pDIO)
		return 0xFFFFFFFF;

	switch (pDIO->dwIoControlCode)
	{
		case DIOC_OPEN:
		{
			Debug_Printf ("DIOC_OPEN\r\n");
			break;
		}
		case DIOC_CLOSEHANDLE:
		{
			Debug_Printf ("DIOC_CLOSEHANDLE\r\n");
			break;
		}
		case IOCTL_NEW_MAPPINGS:
		{
			Debug_Printf ("IOCTL_NEW_MAPPINGS\r\n");

			if (pDIO->cbInBuffer != sizeof(KeyMappings))
			{
				Debug_Printf ("IOCTL_NEW_MAPPINGS - Size mismatch!!!!\r\n");
				return ERROR_BAD_LENGTH;
			}

			memcpy (KeyMappings, (char *)pDIO->lpvInBuffer, sizeof(KeyMappings));

			fMappingOk = TRUE;

			break;
		}
	}
	return 0;
}


//
//	C_KeyboardFilter
//
ULONG C_KeyboardFilter (ULONG ulScanCode)
{
	UCHAR	ucScanCode = (UCHAR)ulScanCode;
	UCHAR	KeyUpFlag;

	if (ucScanCode == 0xE0 || ucScanCode == 0xE1)
	{
		fLastExtended = TRUE;
	}
	else
	{
		if (!fLastExtended)
		{
			// Standard Key

			if (fMappingOk)
			{
				KeyUpFlag = ucScanCode & 0x80;

				ulScanCode = KeyMappings[ucScanCode & 0x7F];

				ulScanCode |= KeyUpFlag;

				Debug_Printf ("C_KeyboardFilter: Mapping %d to %d\r\n", ucScanCode, ulScanCode);
			}
		}
		fLastExtended = FALSE;
	}

    return ulScanCode;
}
				










